"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.uid = uid;
exports.forEach = forEach;
exports.isObj = isObj;
exports.get = get;
exports.cs = cs;
exports.getCache = getCache;
exports.setCache = setCache;
exports.css = css;
exports.keyframes = keyframes;
exports.createSystem = createSystem;
exports.default = void 0;

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _toArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toArray"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

// utils
var ids = {};

function uid() {
  var root = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  ids[root] || (ids[root] = 0);
  return root + (ids[root]++).toString(36);
}

function forEach(obj, sideEffect) {
  Object.keys(obj).forEach(function (key) {
    return sideEffect(obj[key], key);
  });
}

function compose() {
  for (var _len = arguments.length, functions = new Array(_len), _key = 0; _key < _len; _key++) {
    functions[_key] = arguments[_key];
  }

  var valid = functions.filter(function (fn) {
    return typeof fn === 'function';
  });
  return function (param) {
    return valid.reduce(function (composed, fn) {
      return fn(composed);
    }, param);
  };
}

function capitalize(str) {
  return str[0].toUpperCase() + str.slice(1);
}

function kebabCase() {
  var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return str.replace(/[A-Z]/g, '-$&').toLowerCase();
}

function isObj(val) {
  return (0, _typeof2.default)(val) === 'object' && isDef(val) && !Array.isArray(val);
}

function isDef(val) {
  return val === 0 || Boolean(val);
}

function uniq(value, index, arr) {
  return arr.lastIndexOf(value) === index;
}

function get(obj, _ref) {
  var _ref2 = (0, _toArray2.default)(_ref),
      first = _ref2[0],
      after = _ref2.slice(1);

  return isDef(first) && isDef(obj) ? get(obj[first], after) : obj;
}

function set(obj, _ref3, value) {
  var _ref4 = (0, _toArray2.default)(_ref3),
      first = _ref4[0],
      after = _ref4.slice(1);

  if (isDef(first) && isDef(obj)) {
    if (after.length === 0) {
      obj[first] = value;
    } else {
      obj[first] = obj[first] || {};
      set(obj[first], after, value);
    }
  }

  return obj;
}

function merge(target, source) {
  if (target === source) {
    return source;
  }

  var result = target ? merge(null, target) : {};
  forEach(source, function (sourceValue, key) {
    var targetValue = result[key];
    result[key] = isObj(targetValue) && isObj(sourceValue) ? merge(targetValue, sourceValue) : sourceValue;
  });
  return result;
}

function cs() {
  for (var _len2 = arguments.length, classNames = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    classNames[_key2] = arguments[_key2];
  }

  return classNames.filter(isDef).map(String).map(function (v) {
    return v.startsWith('.') ? v.slice(1) : v;
  }).filter(uniq).join(' ');
} // cache


var cache = {};

function resolvePath(value, prop, pseudo, children, wrap) {
  var wrapPath = wrap ? ['$wrap', wrap] : [];
  var pseudoPath = pseudo ? ['$pseudo', pseudo] : [];
  var childPath = children ? ['$children', children] : [];
  var propPath = prop && isDef(value) ? [prop, value] : null;
  return propPath && wrapPath.concat(pseudoPath, childPath, propPath);
}

function getCache(value, prop, pseudo, children, wrap) {
  var path = resolvePath(value, prop, pseudo, children, wrap);
  return path ? get(cache, path) : cache;
}

function setCache(className, value, prop, pseudo, children, wrap) {
  var path = resolvePath(value, prop, pseudo, children, wrap);
  return path ? set(cache, path, className) : cache = className;
} // css


var stylesheet = document.createElement('style');
document.head.append(stylesheet);

function toSelector(className) {
  var pseudo = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  var children = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
  var wrap = arguments.length > 3 ? arguments[3] : undefined;
  var dotted = '.' + className;
  var specific = wrap ? dotted + dotted : dotted;
  var parent = specific + pseudo;
  return Boolean(children) ? children.split(',').map(function (child) {
    return "".concat(parent, " ").concat(child.trim());
  }).join(', ') : parent;
}

function toRule(className, value, prop, pseudo, children, wrap) {
  var selector = toSelector(className, pseudo, children, wrap);
  var rule = "".concat(selector, " { ").concat(prop, ": ").concat(value, "; }");
  return wrap ? "".concat(wrap, " { ").concat(rule, " }") : rule;
}

function insertRule(value, prop, pseudo, children, wrap) {
  var className = uid('r');
  var rule = toRule(className, value, prop, pseudo, children, wrap);
  stylesheet.sheet.insertRule(rule, stylesheet.sheet.cssRules.length);
  return className;
}

function findOrCreateRule(value, prop, pseudo, children, wrap) {
  if (!getCache(value, prop, pseudo, children, wrap)) {
    var className = insertRule(value, prop, pseudo, children, wrap);
    setCache(className, value, prop, pseudo, children, wrap);
  }

  return getCache(value, prop, pseudo, children, wrap);
}

function css(style, pseudo, children, wrap) {
  return Object.keys(style).filter(function (prop) {
    return prop in document.body.style;
  }).map(function (prop) {
    return findOrCreateRule(style[prop], prop, pseudo, children, wrap);
  });
} // keyframes


function toStep(props, step) {
  var css = [];
  forEach(props, function (v, p) {
    return css.push("".concat(p, ": ").concat(v, ";"));
  });
  return "".concat(step, " { ").concat(css.join(' '), " }");
}

function keyframes(steps) {
  var name = uid('k');
  var css = [];
  forEach(steps, function (p, s) {
    return css.push(toStep(p, s));
  });
  var rule = "@keyframes ".concat(name, " { ").concat(css.join(' '), " }");
  stylesheet.sheet.insertRule(rule);
  return name;
} // system


var META = ['$wrap', '$pseudo', '$children'];

function getName() {
  var _ref5 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      displayName = _ref5.displayName,
      As = _ref5.as;

  var asDName = As && As.displayName;
  var asName = As && As.name;
  var tag = typeof As === 'string' && As;
  var name = capitalize(displayName || asDName || asName || tag || 'div');
  return name[0] === 'Σ' ? name : 'Σ' + name;
}

function separateStyle(props, rules) {
  var style = {};
  var other = {};
  forEach(props, function (value, prop) {
    if (rules[prop] || META.includes(prop)) {
      style[prop] = value;
    } else {
      other[prop] = value;
    }
  });
  return [style, other];
} // execute the functional default props by passing them current props


function applyPropsToDefaults(defaults, props) {
  var computedDefaults = {};
  forEach(defaults, function (value, prop) {
    computedDefaults[prop] = prop !== 'as' && typeof value === 'function' ? value(props) : value;
  });
  return computedDefaults;
} // for each style prop, check if the matching rule is a function and execute it


function applyRulesToStyle(style, props, rules, omit) {
  var computedProps = {};
  forEach(style, function (value, prop) {
    if (prop === omit) {
      return computedProps[prop] = value;
    }

    var rule = rules[prop];
    var isRuleFn = typeof rule === 'function';
    var result = isRuleFn ? rule(value, props) : value;
    var isResultObj = isObj(result);

    if (isResultObj) {
      // apply deeper but ignore self referencing rules
      result = applyRulesToStyle(result, props, rules, prop);
    }

    if (isRuleFn && isResultObj) {
      computedProps = merge(computedProps, result);
    } else {
      computedProps[prop] = result;
    }
  });
  return computedProps;
}

function getMetaArgs(type, meta, pseudo, children, wrap) {
  switch (type) {
    case '$wrap':
      return [pseudo, children, meta];

    case '$pseudo':
      return [meta, children, wrap];

    case '$children':
      return [pseudo, meta, wrap];
  }
}

function getClassNames(style, rules, pseudo, children, wrap) {
  var computedStyle = {};
  var classNames = [];
  forEach(style, function (value, prop) {
    // for meta rules go deeper except inside children rules
    if (META.includes(prop) && isObj(value) && !Boolean(children)) {
      forEach(value, function (metaStyle, meta) {
        var metaArgs = getMetaArgs(prop, meta, pseudo, children, wrap);
        var metaClassNames = getClassNames.apply(void 0, [metaStyle, rules].concat((0, _toConsumableArray2.default)(metaArgs)));
        classNames.push.apply(classNames, (0, _toConsumableArray2.default)(metaClassNames));
      });
    } else if (isDef(value)) {
      var rule = typeof rules[prop] === 'string' ? rules[prop] : prop;
      computedStyle[kebabCase(rule)] = value;
    }
  });
  return css(computedStyle, pseudo, children, wrap).concat(classNames);
}

function buildAnimation(steps, props, rules) {
  var computedSteps = {};
  forEach(steps, function (style, step) {
    computedSteps[step] = {};
    var computedProps = applyRulesToStyle(style, props, rules);
    forEach(computedProps, function (value, prop) {
      var rule = typeof rules[prop] === 'string' ? rules[prop] : prop;
      computedSteps[step][kebabCase(rule)] = value;
    });
  });
  return keyframes(computedSteps);
}

function createSystem(renderer) {
  return function system() {
    var rules = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    function createComponent() {
      var defaults = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var wrap = arguments.length > 1 ? arguments[1] : undefined;
      var uniqueClassName = uid('c');

      function applySystem(props) {
        var computedDefaults = applyPropsToDefaults(defaults, props);
        var allProps = merge(computedDefaults, props);

        var _separateStyle = separateStyle(allProps, rules),
            _separateStyle2 = (0, _slicedToArray2.default)(_separateStyle, 2),
            styleProps = _separateStyle2[0],
            passedProps = _separateStyle2[1];

        var computedProps = applyRulesToStyle(styleProps, allProps, rules);

        var _separateStyle3 = separateStyle(computedProps, rules),
            _separateStyle4 = (0, _slicedToArray2.default)(_separateStyle3, 2),
            style = _separateStyle4[0],
            ruleProps = _separateStyle4[1];

        var computedClassNames = getClassNames(style, rules);
        var classNames = [uniqueClassName].concat(computedClassNames);

        var _merge = merge(ruleProps, passedProps),
            Rendered = _merge.as,
            nextProps = (0, _objectWithoutProperties2.default)(_merge, ["as"]);

        return [Rendered || 'div', classNames, nextProps];
      }

      var System = renderer(applySystem);
      System = wrap ? wrap(System) : System;

      System.with = function (props) {
        return createComponent(merge(defaults, props), wrap);
      };

      System.compose = function () {
        for (var _len3 = arguments.length, next = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
          next[_key3] = arguments[_key3];
        }

        return createComponent(defaults, compose.apply(void 0, [wrap].concat(next)));
      }; // prettier-ignore


      System.system = function (extra) {
        return system(merge(rules, extra)).compose(wrap).with(defaults);
      }; // prettier-ignore


      System.toString = function () {
        return '.' + uniqueClassName;
      };

      System.displayName = getName(defaults);

      System.animate = function (timing, steps) {
        var computedAnim = buildAnimation(steps, defaults, rules);
        var animation = (defaults.animation ? defaults.animation + ', ' : '') + computedAnim + ' ' + timing; // prettier-ignore

        var Animated = createComponent(merge(defaults, {
          animation: animation
        }), wrap);
        Animated.animationName = computedAnim;
        return Animated;
      };

      return System;
    }

    return createComponent();
  };
}

var _default = createSystem;
exports.default = _default;
//# sourceMappingURL=index.js.map